class LecturesController < ApplicationController

  def index
    @course = Course.find( params[:course_id] )
    @lectures = @course.lectures.sort_by { |l| l.lecture_number }
    if params[:sord] && params[:sord]=="desc"
      @lectures.reverse!
    end
    respond_to do |format|
      format.html # index.html.erb
      format.xml  { render :partial => "index.xml.builder", :layout => false }
      format.json { render :json => Lecture.to_json( @lectures )}
    end
  end

  def show
    @course = Course.find( params[:course_id] )
    @lecture = Lecture.find( params[:id] )
    respond_to do |format|
      format.html # show.html.erb
      format.xml  { render :xml => @lecture }
      format.json { render :json => @lecture }
    end
  end

  def new
    @course = Course.find( params[:course_id] )
    @lecture = Lecture.new({ :course_id => params[:course_id] })
    respond_to do |format|
      format.html # new.html.erb
      format.xml  { render :xml => @lecture }
      format.json { render :json => @lecture }
    end
  end

  def edit
    @course = Course.find( params[:course_id] )
    @lecture = Lecture.find( params[:id] )
  end

  def create
    @course = Course.find( params[:course_id] )
    @lecture = nil

    begin
      Lecture.transaction do

        record_complete = true

        # Gather any dates to be removed from the temporary lecture object
        lecture_dates_to_delete = []
        if params[:lecture][:lecture_dates_to_delete]
          lecture_dates_to_delete = params[:lecture][:lecture_dates_to_delete].keys
          params[:lecture].delete :lecture_dates_to_delete
          record_complete = false
        end

        # Gather existing dates for the temporary lecture object
        lecture_date_array = []
        if params[:lecture][:existing_lecture_date_attributes]
          existing_lecture_dates = params[:lecture][:existing_lecture_date_attributes]
          existing_lecture_dates.each_pair do |k,v|
            if !lecture_dates_to_delete.index k
              ld = LectureDate.new v
              lecture_date_array.push ld
            end
          end
          params[:lecture].delete :existing_lecture_date_attributes
        end

        # Add a new date to the temporary lecture object if requested
        if params[:lecture][:new_lecture_date_attributes]
          if params[:commit] == 'Add'
            ld = LectureDate.new params[:lecture][:new_lecture_date_attributes]
            lecture_date_array.push ld
            record_complete = false
          end
          params[:lecture].delete :new_lecture_date_attributes
        end

        # Form the temporary lecture object
        @lecture = Lecture.new(params[:lecture])
        @lecture.lecture_dates = lecture_date_array
        @lecture.course_id = @course.id

        respond_to do |format|
          if !record_complete
            # User is still editing
            format.html { number_lecture_dates( @lecture ) ; render :action => "new" }
            format.xml  { raise "Invalid request" }
            format.json { raise "Invalid request" }
          else
            # Raise exception if save fails
            @lecture.save!
            @lecture.lecture_dates.each { |lds| lds.save! }
            flash[:notice] = 'Lecture was successfully created.'
            format.html { redirect_to([@course,@lecture]) }
            format.xml  { render :xml => @lecture, :status => :created, :location => @lecture }
            format.json { render :json => @lecture, :status => :created, :location => @lecture }
          end
        end
      end
    rescue
      respond_to do |format|
        format.html { number_lecture_dates( @lecture ) ; render :action => "new" }
        format.xml  { render :xml => @lecture.errors, :status => :unprocessable_entity }
        format.json { render :json => @lecture.errors, :status => :unprocessable_entity }
      end
    end
  end

  def update
    @lecture = Lecture.find(params[:id])
    @course = @lecture.course
    
    fix_param :topics
    fix_param :readings

    begin
      Lecture.transaction do

        update_complete = true
      
        if params[:lecture][:lecture_dates_to_delete]
          lecture_dates_to_delete = params[:lecture][:lecture_dates_to_delete].keys
          lecture_dates_to_delete.each do |ld|
            @lecture.lecture_dates.find( ld ).destroy
          end
          params[:lecture].delete :lecture_dates_to_delete
          update_complete = false
        end
  
        if params[:lecture][:existing_lecture_date_attributes]
          existing_lecture_dates = params[:lecture][:existing_lecture_date_attributes]
          existing_lecture_dates.each_pair do |k,v|
            if LectureDate.exists?( k )
              @lecture.lecture_dates.find( k ).update_attributes!( v )
            end
          end
          params[:lecture].delete :existing_lecture_date_attributes
        end
      
        if params[:lecture][:new_lecture_date_attributes]
          if params[:commit] == 'Add'
            new_lecture_date = LectureDate.new(params[:lecture][:new_lecture_date_attributes])
            new_lecture_date.course_id = @lecture.course_id
            new_lecture_date.lecture_number = @lecture.lecture_number
            new_lecture_date.lecture_id = @lecture.id
            new_lecture_date.save
            update_complete = false
          end
          params[:lecture].delete :new_lecture_date_attributes
        end
  
        if params[:lecture][:lecture_dates]
          @lecture.lecture_dates.each { |ld| ld.destroy }
          params[:lecture][:lecture_dates].split(',').each do |ld|
            @lecture.add_lecture_date ld
          end
          params[:lecture].delete :lecture_dates
        end

        respond_to do |format|
          if @lecture.update_attributes!(params[:lecture])
            flash[:notice] = 'Lecture was successfully updated.'
            format.html { if update_complete : redirect_to([@course,@lecture]) else render :action => "edit" ; end }
            format.xml  { render :xml => @lecture }
            format.json { head :ok }
          end
        end
      end      
    rescue
      respond_to do |format|
        format.html { render :action => "edit" }
        format.xml  { render :xml => @lecture.errors, :status => :unprocessable_entity }
        format.json { render :json => @lecture.errors, :status => :unprocessable_entity }
      end
    end
  end

  def destroy
    @lecture = Lecture.find(params[:id])
    @lecture.destroy

    respond_to do |format|
      format.html { redirect_to( course_lectures_url( params[:course_id] )) }
      format.xml  { head :ok }
      format.json { head :ok }
    end
  end
  
  private
  
  def number_lecture_dates( lecture )
    lecture_date_index = 1
    lecture.lecture_dates.each do |ld|
      ld.id = lecture_date_index
      lecture_date_index += 1
    end
  end
  
  def fix_param( param )
    if params[param]
      if !params[:lecture]
        params[:lecture] = {}
      end
      params[:lecture][param] = params[param]
      params.delete param
    end
  end
end
