class Course < ActiveRecord::Base
  has_many :lectures, :dependent => :destroy
  has_many :lecture_dates
  
  # Get all courses and sort by time
  def self.find_all_sorted
    Course.find(:all).sort_by { |c| c.date }.reverse
  end
  
  # Convert an array of courses to json format
  def self.to_json( courses )
    courses_hash = courses.collect do |c|
      { :id => c.id, :name => c.name, :code => c.code, :year => c.year, :semester => c.semester }
    end
    courses_hash.to_json
  end
  
  def to_json( )
    attribs = self.attributes
    # Modify legacy schema fields
    attribs.delete "course_id"
    attribs[:id] = self.id
    # Remove non-public fields
    attribs.delete "root_repo"
    # Add lecture ids
    attribs[:lectures] = self.lectures.collect do |l|
      { :id => l.id, :lecture_number => l.lecture_number }
    end
    attribs.to_json
  end
  
  def date
    sem = self.semester
    if sem == "Summer"
      sem = "June"
    elsif sem == "1"
      sem = "January"
    end
    DateTime.parse "#{sem} #{self.year}"
  end

end
