class Lecture < ActiveRecord::Base
  belongs_to :course
  has_many :lecture_dates, :dependent => :destroy
  attr_protected :course_id

  def validate
    @lecture = Lecture.find_by_course_id_and_lecture_number( self.course_id, self.lecture_number )
    if @lecture && @lecture.id != self.id
      errors.add_to_base "Lecture with this number already exists for this course"
    end
  end
  
  def self.to_json( lectures )
    lectures_hash = lectures.collect do |l|
      { :id => l.id,
        :lecture_number => l.lecture_number,
        :topics => l.topics,
        :readings => l.readings,
        :lecture_dates => l.lecture_dates.collect { |d| d.date }}
    end
    { :identifier => 'id', :label => 'id', :items => lectures_hash }.to_json
  end

  def to_json
    attribs = self.attributes
    attribs.to_json
  end
  
  def add_lecture_date( date )
    lecture_date = LectureDate.new :lecture_number => self.lecture_number, :date => date
    lecture_date.course_id = self.course_id
    lecture_date.lecture_id = self.id
    lecture_date.save
  end
end
